@@ -14,6 +14,7 @@
 #undef USING_OAF
 #undef ENABLE_GB
 #undef HAVE_IEEEFP_H
+#undef HAVE_IEEE754_H
 #undef ENABLE_GNOME
 #undef HAVE_PSPELL
 #undef ENABLE_EVOLUTION
@@ -23,11 +24,3 @@
 #undef USE_WM_ICONS
 
 @BOTTOM@
-/* This is from libglade */
-
-#ifdef DEBUG
-#  define debug(stmnt) stmnt
-#else
-#  define debug(stmnt) /* nothing */
-#endif
-
