@@ -3,32 +3,30 @@
  * See: S59E2B.HTM
  *
  * Author:
- *    Michael Meeks (michael@imaginator.com)
+ *    Michael Meeks (michael@ximian.com)
  *
- * (C) 1998, 1999, 2000 Michael Meeks
+ * (C) 1998-2001 Michael Meeks
  */
 
+#include <gnumeric-config.h>
+#include <gnumeric.h>
+#include "ms-formula-write.h"
+#include "excel.h"
+#include "ms-biff.h"
+#include "formula-types.h"
+#include "boot.h"
+#include "gutils.h"
+#include "func.h"
+#include "value.h"
+#include "expr-name.h"
+#include "str.h"
+#include "parse-util.h"
+
 #include <fcntl.h>
 #include <assert.h>
 #include <stdio.h>
 
-#include <config.h>
-#include <gnome.h>
-
-#include "gnumeric.h"
-#include "gutils.h"
-#include "func.h"
-#include "value.h"
-
-#include "parse-util.h"
-
-#include "excel.h"
-#include "ms-biff.h"
-#include "ms-formula-write.h"
-#include "formula-types.h"
-
 #define FORMULA_DEBUG 0
-extern int ms_excel_formula_debug;
 /*#define DO_IT (ms_excel_formula_debug > 0)*/
 #define DO_IT (1)
 
@@ -172,12 +170,11 @@ ms_formula_build_pre_data (ExcelSheet *sheet, ExprTree const *tree)
 
 	case OPER_FUNCALL:
 	{
-		GList  *l;
+		ExprList *l;
 		FormulaCacheEntry *fce;
 		const gchar *name = function_def_get_name (tree->func.func);
 
-		for (l = tree->func.arg_list; l;
-		     l = g_list_next (l))
+		for (l = tree->func.arg_list; l; l = l->next)
 			ms_formula_build_pre_data (sheet, l->data);
 
 		fce = get_formula_index (sheet, name);
@@ -341,7 +338,8 @@ write_cellref_v8 (PolishData *pd, const CellRef *ref,
 static void
 write_string (PolishData *pd, const gchar *txt)
 {
-	if (!txt || txt[0] == '\0')
+	/* FIXME : Check this logic.  Why would we pass a NULL ? */
+	if (txt == NULL)
 		push_guint8 (pd, FORMULA_PTG_MISSARG);
 	else {
 		push_guint8 (pd, FORMULA_PTG_STR);
@@ -350,13 +348,11 @@ write_string (PolishData *pd, const gchar *txt)
 }
 
 static void
-write_area (PolishData *pd, const CellRef *a, const CellRef *b)
+write_area (PolishData *pd, CellRef const *a, CellRef const *b)
 {
 	guint8 data[24];
 
-	if (!a->sheet || !b->sheet ||
-	    (a->sheet == pd->sheet->gnum_sheet &&
-	     a->sheet == b->sheet)) {
+	if (a->sheet == NULL && b->sheet == NULL) {
 		push_guint8 (pd, OP_REF (FORMULA_PTG_AREA));
 		if (pd->ver <= MS_BIFF_V7) {
 			write_cellref_v7 (pd, a,
@@ -387,7 +383,12 @@ write_area (PolishData *pd, const CellRef *a, const CellRef *b)
 
 		push_guint8 (pd, OP_REF (FORMULA_PTG_AREA_3D));
 		if (pd->ver <= MS_BIFF_V7) {
-			MS_OLE_SET_GUINT16 (data, 0); /* FIXME ? */
+			/* FIXME for now assume that the external reference is in the same workbook */
+			if (pd->sheet->gnum_sheet->workbook != a->sheet->workbook) {
+				g_warning ("References to external workbooks are not supported yet");
+			}
+
+			MS_OLE_SET_GUINT16 (data, 0xffff);
 			MS_OLE_SET_GUINT32 (data +  2, 0x0);
 			MS_OLE_SET_GUINT32 (data +  6, 0x0);
 			MS_OLE_SET_GUINT16 (data + 10, first_idx);
@@ -457,7 +458,7 @@ write_ref (PolishData *pd, const CellRef *ref)
 static void
 write_funcall (PolishData *pd, FormulaCacheEntry *fce, ExprTree *tree)
 {
-	GList   *args     = tree->func.arg_list;
+	ExprList *args     = tree->func.arg_list;
 	gint     num_args = 0;
 	gboolean prompt   = 0;
 	gboolean cmdequiv = 0;
@@ -478,9 +479,8 @@ write_funcall (PolishData *pd, FormulaCacheEntry *fce, ExprTree *tree)
 		}
 	}
 
-	while (args) {
+	for (; args ; args = args->next) {
 		write_node (pd, args->data, 0);
-		args = g_list_next (args);
 		num_args++;
 	}
 
@@ -533,10 +533,11 @@ write_node (PolishData *pd, ExprTree *tree, int paren_level)
 		{ 0, 0, 0, 0 }, /* Name     */
 		{ 0, 0, 0, 0 }, /* Constant */
 		{ 0, 0, 0, 0 }, /* Var      */
-		{ FORMULA_PTG_U_PLUS,	 5, 0, 0 }, /* Unary - */
-		{ FORMULA_PTG_U_MINUS,	 5, 0, 0 }, /* Unary + */
+		{ FORMULA_PTG_U_MINUS,	 5, 0, 0 }, /* Unary - */
+		{ FORMULA_PTG_U_PLUS,	 5, 0, 0 }, /* Unary + */
 		{ FORMULA_PTG_PERCENT,	 5, 0, 0 }, /* Percentage (NOT MODULO) */
-		{ 0, 0, 0, 0 }  /* Array    */
+		{ 0, 0, 0, 0 }, /* Array    */
+		{ 0, 0, 0, 0 }  /* Set      */
 	};
 	int op;
 	g_return_if_fail (pd);
@@ -697,7 +698,7 @@ write_node (PolishData *pd, ExprTree *tree, int paren_level)
 
 		for (idx = 0; idx < pd->sheet->wb->names->len; idx++)
 			if (!strcmp(tree->name.name->name->str,
-				    g_ptr_array_index (pd->sheet->wb->names, idx))) {
+				    (char *) g_ptr_array_index (pd->sheet->wb->names, idx))) {
 
 			    MS_OLE_SET_GUINT8  (data + 0, FORMULA_PTG_NAME);
 			    MS_OLE_SET_GUINT16 (data + 1, idx + 1);
@@ -707,7 +708,19 @@ write_node (PolishData *pd, ExprTree *tree, int paren_level)
 		break;
 	}
 
-	case OPER_ARRAY:
+	case OPER_ARRAY: {
+		ExprArray const *array = &tree->array;
+		guint8 data[5];
+		MS_OLE_SET_GUINT8 (data, FORMULA_PTG_EXPR);
+		MS_OLE_SET_GUINT16 (data+1, pd->row - array->y);
+		MS_OLE_SET_GUINT16 (data+3, pd->col - array->x);
+		ms_biff_put_var_write (pd->bp, data, 5);
+
+		/* Be anal */
+		g_return_if_fail (paren_level == 0);
+		break;
+	}
+
 	default:
 	{
 		gchar *err = g_strdup_printf ("Unknown Operator %d",
@@ -762,7 +775,7 @@ guint32
 ms_excel_write_formula (BiffPut *bp, ExcelSheet *sheet, ExprTree *expr,
 			int fn_col, int fn_row, int paren_level)
 {
-	PolishData *pd;
+	PolishData pd;
 	MsOlePos start;
 	guint32 len;
 
@@ -770,27 +783,24 @@ ms_excel_write_formula (BiffPut *bp, ExcelSheet *sheet, ExprTree *expr,
 	g_return_val_if_fail (expr, 0);
 	g_return_val_if_fail (sheet, 0);
 
-	pd = g_new (PolishData, 1);
-	pd->col    = fn_col;
-	pd->row    = fn_row;
-	pd->sheet  = sheet;
-	pd->bp     = bp;
-	pd->arrays = NULL;
-	pd->ver    = sheet->wb->ver;
+	pd.col    = fn_col;
+	pd.row    = fn_row;
+	pd.sheet  = sheet;
+	pd.bp     = bp;
+	pd.arrays = NULL;
+	pd.ver    = sheet->wb->ver;
 
 	start = bp->length;
-	write_node (pd, expr, 0);
+	write_node (&pd, expr, 0);
 	len = bp->length - start;
 
-	if (pd->arrays) {
-		push_guint16 (pd, 0x0); /* Sad but true */
-		push_guint8  (pd, 0x0);
+	if (pd.arrays) {
+		push_guint16 (&pd, 0x0); /* Sad but true */
+		push_guint8  (&pd, 0x0);
 
-		while (pd->arrays)
-			write_arrays (pd);
+		while (pd.arrays)
+			write_arrays (&pd);
 	}
 
-	g_free (pd);
-
 	return len;
 }
