@@ -5,10 +5,12 @@
  *  Miguel de Icaza (miguel@gnu.org)
  *  Michael Meeks   (mmeeks@gnu.org)
  *  Morten Welinder (terra@diku.dk)
- *  Jody Goldberg   (jgoldberg@home.org)
+ *  Jody Goldberg   (jody@gnome.org)
  */
-#include <config.h>
+#include <gnumeric-config.h>
+#include "gnumeric.h"
 #include "func.h"
+
 #include "portability.h"
 #include "parse-util.h"
 #include "eval.h"
@@ -17,17 +19,18 @@
 #include "symbol.h"
 #include "workbook.h"
 #include "sheet.h"
+#include "number-match.h"
 
 #include <string.h>
 #include <glib.h>
-#include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
 #include <stdlib.h>
+#include <ctype.h>
 
 /* These are not supported yet */
 typedef enum {
 	FUNCTION_RETURNS_ARRAY = 0x01, /* eg transpose(), mmult() */
-	FUNCTION_RECALC_ALWAYS = 0x02, /* eg now() */
+	FUNCTION_RECALC_ALWAYS = 0x02, /* eg now(), today() */
 
 	/* For functions that are not exactly compatible with various import
 	 * formats.  We need to recalc their results to avoid changing values
@@ -92,11 +95,15 @@ functions_init (void)
 	information_functions_init ();
 }
 
+static void function_def_get_full_info_if_needed (FunctionDefinition *fn_def);
+
 static void
 copy_hash_table_to_ptr_array (gpointer key, gpointer value, gpointer array)
 {
 	Symbol *sym = value;
 	FunctionDefinition *fd = sym->data;
+
+	function_def_get_full_info_if_needed (fd);
 	if (sym->type == SYMBOL_FUNCTION &&
 	    fd->help != NULL && fd->name != NULL)
 		g_ptr_array_add (array, fd);
@@ -276,7 +283,7 @@ extract_arg_types (FunctionDefinition *def)
 
 
 static Value *
-error_function_no_full_info (FunctionEvalInfo *ei, GList *expr_node_list)
+error_function_no_full_info (FunctionEvalInfo *ei, ExprList *expr_node_list)
 {
 	return value_new_error (ei->pos, _("Function implementation not available."));
 }
@@ -473,7 +480,7 @@ function_add_name_only (FunctionCategory *category,
 
 /* Handle unknown functions on import without losing their names */
 static Value *
-unknownFunctionHandler (FunctionEvalInfo *ei, GList *expr_node_list)
+unknownFunctionHandler (FunctionEvalInfo *ei, ExprList *expr_node_list)
 {
 	return value_new_error (ei->pos, gnumeric_err_NAME);
 }
@@ -620,11 +627,11 @@ static inline Value *
 function_marshal_arg (FunctionEvalInfo *ei,
 		      ExprTree         *t,
 		      char              arg_type,
-		      gboolean         *type_mismatch)
+		      Value            **type_mismatch)
 {
 	Value *v;
 
-	*type_mismatch = FALSE;
+	*type_mismatch = NULL;
 
 	/*
 	 *  This is so we don't dereference 'A1' by accident
@@ -657,10 +664,27 @@ function_marshal_arg (FunctionEvalInfo *ei,
 				break;
 		}
 
+		if (v->type == VALUE_STRING) {
+			Value *newv = format_match_number (value_peek_string (v),
+							   NULL,
+							   NULL);
+			value_release (v);
+			v = newv;
+			break;
+		}
+
+		if (v->type == VALUE_ERROR) {
+			*type_mismatch = v;
+			v = NULL;
+			break;
+		}
+
 		if (v->type != VALUE_INTEGER &&
 		    v->type != VALUE_FLOAT &&
-		    v->type != VALUE_BOOLEAN)
-			*type_mismatch = TRUE;
+		    v->type != VALUE_BOOLEAN) {
+			*type_mismatch = value_new_error (ei->pos,
+							  gnumeric_err_VALUE);
+		}
 		break;
 
 	case 's':
@@ -674,14 +698,20 @@ function_marshal_arg (FunctionEvalInfo *ei,
 				break;
 		}
 
-		if (v->type != VALUE_STRING)
-			*type_mismatch = TRUE;
+		if (v->type == VALUE_ERROR) {
+			*type_mismatch = v;
+			v = NULL;
+		} else if (v->type != VALUE_STRING) {
+			*type_mismatch = value_new_error (ei->pos,
+							  gnumeric_err_VALUE);
+		}
 		break;
 
 	case 'r':
-		if (v->type != VALUE_CELLRANGE)
-			*type_mismatch = TRUE;
-		else {
+		if (v->type != VALUE_CELLRANGE) {
+			*type_mismatch = value_new_error (ei->pos,
+							  gnumeric_err_VALUE);
+		} else {
 			cellref_make_abs (&v->v_range.cell.a,
 					  &v->v_range.cell.a,
 					  ei->pos);
@@ -692,14 +722,18 @@ function_marshal_arg (FunctionEvalInfo *ei,
 		break;
 
 	case 'a':
-		if (v->type != VALUE_ARRAY)
-			*type_mismatch = TRUE;
+		if (v->type != VALUE_ARRAY) {
+			*type_mismatch = value_new_error (ei->pos,
+							  gnumeric_err_VALUE);
+		}
 		break;
 
 	case 'A':
 		if (v->type != VALUE_ARRAY &&
-		    v->type != VALUE_CELLRANGE)
-			*type_mismatch = TRUE;
+		    v->type != VALUE_CELLRANGE) {
+			*type_mismatch = value_new_error (ei->pos,
+							  gnumeric_err_VALUE);
+		}
 
 		if (v->type == VALUE_CELLRANGE) {
 			cellref_make_abs (&v->v_range.cell.a,
@@ -744,14 +778,14 @@ free_values (Value **values, int top)
 /**
  * function_call_with_list:
  * @ei: EvalInfo containing valid fn_def!
- * @args: GList of ExprTree args.
+ * @args: ExprList of ExprTree args.
  *
  * Do the guts of calling a function.
  *
  * Return value:
  **/
 Value *
-function_call_with_list (FunctionEvalInfo *ei, GList *l)
+function_call_with_list (FunctionEvalInfo *ei, ExprList *l)
 {
 	FunctionDefinition const *fn_def;
 	int argc, arg;
@@ -769,7 +803,7 @@ function_call_with_list (FunctionEvalInfo *ei, GList *l)
 		return fn_def->fn.fn_nodes (ei, l);
 
 	/* Functions that take pre-computed Values */
-	argc = g_list_length (l);
+	argc = expr_list_length (l);
 	if (argc > fn_def->fn.args.max_args ||
 	    argc < fn_def->fn.args.min_args) {
 		return value_new_error (ei->pos,
@@ -779,17 +813,20 @@ function_call_with_list (FunctionEvalInfo *ei, GList *l)
 	values = g_new (Value *, fn_def->fn.args.max_args);
 
 	for (arg = 0; l; l = l->next, ++arg) {
-		char     arg_type;
-		gboolean type_mismatch;
+		char  arg_type;
+		Value *type_mismatch;
 
 		arg_type = fn_def->fn.args.arg_types[arg];
 
-		values [arg] = function_marshal_arg (ei, l->data, arg_type,
+		values[arg] = function_marshal_arg (ei, l->data, arg_type,
 						     &type_mismatch);
 
-		if (type_mismatch || values [arg] == NULL) {
+		if (type_mismatch || values[arg] == NULL) {
 			free_values (values, arg + 1);
-			return value_new_error (ei->pos, gnumeric_err_VALUE);
+			if (type_mismatch)
+				return type_mismatch;
+			else
+				return value_new_error (ei->pos, gnumeric_err_VALUE);
 		}
 	}
 	while (arg < fn_def->fn.args.max_args)
@@ -841,7 +878,7 @@ function_def_call_with_values (EvalPos const *ep,
 		 * temporary ExprNodes with constants.
 		 */
 		ExprConstant *tree = NULL;
-		GList *l = NULL;
+		ExprList *l = NULL;
 		int i;
 
 		if (argc) {
@@ -853,7 +890,7 @@ function_def_call_with_values (EvalPos const *ep,
 				tree [i].ref_count = 1;
 				tree [i].value = values [i];
 
-				l = g_list_append (l, &(tree [i]));
+				l = expr_list_append (l, &(tree [i]));
 			}
 		}
 
@@ -861,7 +898,7 @@ function_def_call_with_values (EvalPos const *ep,
 
 		if (tree) {
 			g_free (tree);
-			g_list_free (l);
+			expr_list_free (l);
 		}
 
 	} else
@@ -979,7 +1016,7 @@ function_iterate_do_value (EvalPos      const *ep,
  * @fp:               The position in a workbook at which to evaluate
  * @callback:         The routine to be invoked for every value computed
  * @callback_closure: Closure for the callback.
- * @expr_node_list:   a GList of ExprTrees (what a Gnumeric function would get).
+ * @expr_node_list:   a ExprList of ExprTrees (what a Gnumeric function would get).
  * @strict:           If TRUE, the function is considered "strict".  This means
  *                   that if an error value occurs as an argument, the iteration
  *                   will stop and that error will be returned.  If FALSE, an
@@ -998,12 +1035,12 @@ function_iterate_do_value (EvalPos      const *ep,
  * Value found on the list (this means that ranges get properly expaned).
  */
 Value *
-function_iterate_argument_values (const EvalPos      *ep,
-				  FunctionIterateCB callback,
-				  void                    *callback_closure,
-				  GList                   *expr_node_list,
-				  gboolean                strict,
-				  gboolean		  ignore_blank)
+function_iterate_argument_values (EvalPos const		*ep,
+				  FunctionIterateCB	 callback,
+				  void			*callback_closure,
+				  ExprList		*expr_node_list,
+				  gboolean		 strict,
+				  gboolean		 ignore_blank)
 {
 	Value * result = NULL;
 
@@ -1062,7 +1099,16 @@ tokenized_help_new (FunctionDefinition const *fn_def)
 			if (ptr[0] == '\\' && ptr[1])
 				ptr += 2;
 
-			if (*ptr == '@' && seek_at && last_newline) {
+			/* FIXME : This is hugely ugly.  we need a decent
+			 * format for this stuff.
+			 * @SECTION=content is damn ugly considering we
+			 * are saying things like @r = bob in side the content.
+			 * for now make the assumption that any args will
+			 * always start with lower case.
+			 */
+			if (*ptr == '@' && isupper (*(unsigned char *)(ptr +1)) &&
+			    seek_at && last_newline) {
+				/* previous newline if this is not the first */
 				if (ptr != start)
 					*(ptr-1) = '\0';
 				else
