@@ -149,7 +149,7 @@ gnumeric_date2unix (FunctionEvalInfo *ei, Value **argv)
 	if (fabs (fserial - serial) >= 1.0 || utime == (time_t)-1)
 		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
-	return value_new_float (utime + (fserial - serial));
+	return value_new_float (utime + DAY_SECONDS * (fserial - serial));
 }
 
 /***************************************************************************/
@@ -338,8 +338,8 @@ gnumeric_datedif (FunctionEvalInfo *ei, Value **argv)
 		}
 	}
 
-	g_date_free (gdate1);
-	g_date_free (gdate2);
+	datetime_g_free (gdate1);
+	datetime_g_free (gdate2);
 
 	return result;
 }
@@ -379,7 +379,7 @@ gnumeric_edate (FunctionEvalInfo *ei, Value **argv)
 	date = datetime_serial_to_g (serial);
 
 	if (!g_date_valid (date)) {
-                  g_date_free (date);
+                  datetime_g_free (date);
                   return value_new_error (ei->pos, gnumeric_err_VALUE);
 	}
 
@@ -389,12 +389,12 @@ gnumeric_edate (FunctionEvalInfo *ei, Value **argv)
 	        g_date_subtract_months (date, -months);
 
 	if (!g_date_valid (date)) {
-                  g_date_free (date);
+                  datetime_g_free (date);
                   return value_new_error (ei->pos, gnumeric_err_NUM);
 	}
 
 	res = value_new_int (datetime_g_to_serial (date));
-	g_date_free (date);
+	datetime_g_free (date);
 	return res;
 }
 
@@ -860,8 +860,8 @@ gnumeric_days360 (FunctionEvalInfo *ei, Value **argv)
 	result = ((year2 - year1) * 12 + (month2 - month1)) * 30 +
 		(day2 - day1);
 
-	g_date_free (date1);
-	g_date_free (date2);
+	datetime_g_free (date1);
+	datetime_g_free (date2);
 
 	return value_new_int (flipped ? -result : result);
 }
@@ -997,7 +997,7 @@ static char * help_networkdays = {
 	   "NETWORKDAYS returns the number of non-weekend non-holidays between "
 	   "@start_date and @end_date.  Holidays optionally supplied in @holidays."
 	   "\n"
-	   "Returns #NUM if start_date or end_date are invalid.\n"
+	   "Returns #NUM! if start_date or end_date are invalid.\n"
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -1025,7 +1025,7 @@ get_serial_weekday (int serial, int * offset)
 			*offset = 4;
 	} else
 		serial = -1;
-	g_date_free (date);
+	datetime_g_free (date);
 	return serial;
 }
 
@@ -1061,17 +1061,18 @@ networkdays_holiday_callback(EvalPos const *ep,
 	} else
 		res = value_new_error (ep, gnumeric_err_NUM);
 
-	g_date_free (date);
+	datetime_g_free (date);
 	return res;
 }
 
 static Value *
 gnumeric_networkdays (FunctionEvalInfo *ei, Value **argv)
 {
-	int start_serial = datetime_value_to_serial (argv[0]);
-	int end_serial = datetime_value_to_serial (argv[1]);
+	int start_serial;
+	int end_serial;
 	int start_offset, end_offset, res;
 	networkdays_holiday_closure close;
+	GDate * start_date;
 
 	if (argv[0]->type == VALUE_ERROR)
 		return value_duplicate (argv[0]);
@@ -1088,6 +1089,7 @@ gnumeric_networkdays (FunctionEvalInfo *ei, Value **argv)
 		end_serial = tmp;
 	}
 
+	start_date = datetime_serial_to_g (start_serial);
 	close.start_serial = start_serial;
 	close.end_serial = end_serial;
 	close.res = 0;
@@ -1107,7 +1109,112 @@ gnumeric_networkdays (FunctionEvalInfo *ei, Value **argv)
 				    &close);
 	}
 
-	return value_new_int (res - start_offset + end_offset + 1 - close.res);
+	res = res - start_offset + end_offset - close.res;
+
+	if (g_date_weekday (start_date) < G_DATE_SATURDAY)
+		res++;
+	datetime_g_free (start_date);
+
+	return value_new_int (res);
+}
+
+/***************************************************************************/
+
+static char *help_isoweeknum = {
+	N_("@FUNCTION=ISOWEEKNUM\n"
+	   "@SYNTAX=ISOWEEKNUM (date)\n"
+
+	   "@DESCRIPTION="
+	   "ISOWEEKNUM returns the ISO 8601 week number of @date."
+	   "\n"
+	   "Returns #NUM! if date is invalid."
+	   "\n"
+	   "An ISO 8601 week starts on Monday. Weeks are numbered from 1. A "
+	   "week including days from two different years is assigned to the "
+	   "year which includes the most days. This means that Dec 31 could "
+	   "be in week 1 of the following year, and Jan 1 could be in week 52 "
+	   "or 53 of the previous year."
+	   "\n"
+	   "@EXAMPLES=\n"
+	   "If A1 contains 12/21/00 then ISOWEEKNUM(A1)=51"
+	   "\n"
+	   "@SEEALSO=WEEKNUM")
+};
+
+static Value *
+gnumeric_isoweeknum (FunctionEvalInfo *ei, Value **argv)
+{
+	Value *res;
+	GDate *date;
+	int isoweeknum;
+	
+	if (argv[0]->type == VALUE_ERROR)
+		return value_duplicate (argv[0]);
+
+	date = datetime_value_to_g (argv[0]);
+	if (date == NULL || !g_date_valid (date))
+                  return value_new_error (ei->pos, gnumeric_err_VALUE);
+
+	isoweeknum = datetime_weeknum (date, WEEKNUM_METHOD_ISO);
+	res = value_new_int (isoweeknum);
+	
+	g_date_free (date);
+	return res;
+}
+
+/***************************************************************************/
+
+static char *help_weeknum = {
+	N_("@FUNCTION=WEEKNUM\n"
+	   "@SYNTAX=WEEKNUM (date, method)\n"
+
+	   "@DESCRIPTION="
+	   "WEEKNUM returns the week number of @date according to the given "
+	   "@method.\n"
+	   "\n"
+	   "@method defaults to 1.\n"
+	   "For method=1, week starts on Sunday, and days before first Sunday "
+	   "are in week 0.\n"
+	   "For method=2, week starts on Monday, and days before first Monday "
+	   "are in week 0.\n"
+	   "For method=150, the ISO 8601 week number is returned.\n"
+	   "\n"
+	   "Returns #NUM! if date or method is invalid."
+	   "\n"
+	   "This function is Excel compatible, except that Excel does not "
+	   "support ISO 8601 week numbers."
+	   "\n"
+	   "@EXAMPLES=\n"
+	   "If A1 contains 12/21/00 then WEEKNUM(A1,2)=51"
+	   "\n"
+	   "@SEEALSO=ISOWEEKNUM")
+};
+
+static Value *
+gnumeric_weeknum (FunctionEvalInfo *ei, Value **argv)
+{
+	Value *res;
+	GDate *date;
+	int weeknum;
+	int method = argv[1] ? value_get_as_int (argv[1]) : 1;
+
+	if (!(method == WEEKNUM_METHOD_SUNDAY ||
+	      method == WEEKNUM_METHOD_MONDAY ||
+	      method == WEEKNUM_METHOD_ISO))
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
+	
+	if (argv[0]->type == VALUE_ERROR)
+		return value_duplicate (argv[0]);
+
+	date = datetime_value_to_g (argv[0]);
+	if (date == NULL || !g_date_valid (date))
+                  return value_new_error (ei->pos, gnumeric_err_VALUE);
+
+	weeknum = datetime_weeknum (date, method);
+	res = value_new_int (weeknum);
+	
+	g_date_free (date);
+	return res;
 }
 
 /***************************************************************************/
@@ -1208,4 +1315,12 @@ date_functions_init(void)
 	def = function_add_args (cat,  "year",           "S",
 				 "date",
 				 &help_year,        gnumeric_year);
+
+	def = function_add_args (cat,  "isoweeknum",         "S",
+				 "date",
+				 &help_isoweeknum,  gnumeric_isoweeknum);
+
+	def = function_add_args (cat,  "weeknum",         "S|f",
+				 "date",
+				 &help_weeknum,     gnumeric_weeknum);
 }
