@@ -27,6 +27,7 @@
 #include "sheet.h"
 #include "cell.h"
 #include "value.h"
+#include "mathfunc.h"
 #include "number-match.h"
 #include "workbook-control.h"
 
@@ -115,7 +116,6 @@ validation_eval (WorkbookControl *wbc, MStyle const *mstyle,
 	Validation *v;
 	Cell	   *cell;
 	char	   *msg = NULL;
-	EvalPos     ep;
 	gboolean    allocated_msg = FALSE;
 	ValidationStatus result;
 
@@ -123,7 +123,7 @@ validation_eval (WorkbookControl *wbc, MStyle const *mstyle,
 		return VALIDATION_STATUS_VALID;
 
 	v = mstyle_get_validation (mstyle);
-	g_return_val_if_fail (v != NULL, 1);
+	g_return_val_if_fail (v != NULL, VALIDATION_STATUS_VALID);
 
 	if (v->style == VALIDATION_TYPE_ANY)
 		return VALIDATION_STATUS_VALID;
@@ -138,46 +138,55 @@ validation_eval (WorkbookControl *wbc, MStyle const *mstyle,
 		msg = g_strdup_printf (_("Cell %s is not permitted to be blank"),
 				       cell_name (cell));
 	} else {
-		Value *val;
-		ExprTree *val_expr, *expr;
-
-		g_return_val_if_fail (cell != NULL, VALIDATION_STATUS_VALID);
-		g_return_val_if_fail (cell->value != NULL, VALIDATION_STATUS_VALID);
-
-		val = cell->value;
-		eval_pos_init_cell (&ep, cell);
+		ExprTree *val_expr = NULL, *expr = NULL;
+		Value *val = cell->value;
 
 		switch (v->type) {
 		case VALIDATION_TYPE_ANY :
 			return VALIDATION_STATUS_VALID;
 
-		case VALIDATION_TYPE_AS_INT : {
-			double dummy;
-			if (val->type == VALUE_FLOAT &&
-			    abs (modf (cell->value->v_float.val, &dummy)) > 1e-10) {
-				msg = g_strdup_printf (_("'%f' is not an integer"),
-						       cell->value->v_float.val);
-				break;
-			}
-		}
-
-		case VALIDATION_TYPE_AS_DATE :	/* What the hell does this do */
-		case VALIDATION_TYPE_AS_TIME :	/* What the hell does this do */
+		case VALIDATION_TYPE_AS_INT :
 		case VALIDATION_TYPE_AS_NUMBER :
-			if (val->type == VALUE_ERROR)
-				msg = g_strdup_printf (_("'%f' is not a number"),
-						       val->v_float.val);
-			else if (cell->value->type == VALUE_STRING) {
-				val = format_match_number (val->v_str.val->str, NULL, NULL);
-				if (val != NULL)
-					val_expr = expr_tree_new_constant (val);
-				else
-					msg = g_strdup_printf (_("'%s' is not a number"),
-							       val->v_str.val->str);
+		case VALIDATION_TYPE_AS_DATE :		/* What the hell does this do */
+		case VALIDATION_TYPE_AS_TIME : {	/* What the hell does this do */
+			Value *res = NULL;
+			/* we know it is not empty */
+			if (val->type == VALUE_ERROR) {
+				msg = g_strdup_printf (_("'%s' is an error"),
+						       val->v_err.mesg->str);
+				break;
+			} else if (cell->value->type == VALUE_STRING) {
+				char const *s = value_peek_string (val);
+				res = format_match_number (s, NULL, NULL);
+				if (res == NULL) {
+					char const *fmt;
+					/* FIXME what else is needed */
+					if (v->type == VALIDATION_TYPE_AS_DATE) {
+						fmt = N_("'%s' is not a valid date");
+					} else if (v->type == VALIDATION_TYPE_AS_TIME) {
+						fmt = N_("'%s' is not a valid time");
+					} else
+						fmt = N_("'%s' is not a number");
+					msg = g_strdup_printf (_(fmt), s);
+					break;
+				}
 			} else
-				val_expr = expr_tree_new_constant (
-					value_duplicate (cell->value));
+				res = value_duplicate (cell->value);
+
+			if (v->type == VALIDATION_TYPE_AS_INT &&
+			    res != NULL && res->type == VALUE_FLOAT) {
+				gnum_float f = value_get_as_float (res);
+				gboolean isint = fabs (f - gnumeric_fake_round (f)) < 1e-10;
+				if (!isint) {
+					char const *valstr = value_peek_string (val);
+					msg = g_strdup_printf (_("'%s' is not an integer"), valstr);
+					break;
+				}
+			}
+
+			val_expr = expr_tree_new_constant (res);
 			break;
+		}
 
 		case VALIDATION_TYPE_IN_LIST :
 #warning TODO
@@ -228,17 +237,24 @@ validation_eval (WorkbookControl *wbc, MStyle const *mstyle,
 
 		if (expr != NULL) {
 			ParsePos  pp;
+			EvalPos   ep;
 			char	 *expr_str;
-			Value    *val = expr_eval (expr, &ep, EVAL_STRICT);
-			gboolean  dummy, valid = value_get_as_bool (val, &dummy);
+			Value    *val;
+			gboolean  dummy, valid;
+
+			val = expr_eval (expr, eval_pos_init_cell (&ep, cell),
+					 EVAL_STRICT);
+			valid = value_get_as_bool (val, &dummy);
 			value_release (val);
+
 			if (valid && v->op != VALIDATION_OP_BETWEEN) {
 				if (v->type != VALIDATION_TYPE_CUSTOM)
 					expr_tree_unref (expr);
 				return VALIDATION_STATUS_VALID;
 			}
 
-			if (v->op == VALIDATION_OP_BETWEEN || v->op == VALIDATION_OP_NOT_BETWEEN) {
+			if ((v->op == VALIDATION_OP_BETWEEN && valid) ||
+			    v->op == VALIDATION_OP_NOT_BETWEEN) {
 				g_return_val_if_fail (v->expr[1] != NULL, VALIDATION_STATUS_VALID);
 
 				expr_tree_ref (v->expr[1]);
